@@ -20,6 +20,8 @@ from kudu.compat import unittest, long
 from kudu.tests.common import KuduTestBase
 from kudu.client import Partitioning
 import kudu
+import datetime
+from pytz import utc
 
 
 class TestClient(KuduTestBase, unittest.TestCase):
@@ -32,15 +34,16 @@ class TestClient(KuduTestBase, unittest.TestCase):
 
         self.assertEqual(table.name, self.ex_table)
         self.assertEqual(table.num_columns, len(self.schema))
+        self.assertIsNotNone(table.id)
 
     def test_table_column(self):
         table = self.client.table(self.ex_table)
         cols = [(table['key'], 'key', 'int32'),
                 (table[1], 'int_val', 'int32'),
-                (table[-1], 'string_val', 'string')]
+                (table[-1], 'unixtime_micros_val', 'unixtime_micros')]
 
         for col, name, type in cols:
-            assert col.name == bytes(name)
+            assert col.name == name
             assert col.parent is table
 
             result_repr = repr(col)
@@ -81,7 +84,7 @@ class TestClient(KuduTestBase, unittest.TestCase):
             self.client.delete_table(name)
 
     def test_is_multimaster(self):
-        assert not self.client.is_multimaster
+        assert self.client.is_multimaster
 
     def test_delete_table(self):
         name = "peekaboo"
@@ -129,7 +132,19 @@ class TestClient(KuduTestBase, unittest.TestCase):
 
             self.client.create_table(
                 name, self.schema,
-                partitioning=Partitioning().set_range_partition_columns([]))
+                partitioning=Partitioning()
+                    .set_range_partition_columns(['key'])
+                    .add_range_partition_split({'key': 10})
+                    .add_range_partition_split([20])
+                    .add_range_partition_split((30,))
+            )
+            self.client.delete_table(name)
+
+            self.client.create_table(
+                name, self.schema,
+                partitioning=Partitioning().add_hash_partitions(['key'],
+                                                                2,
+                                                                seed=342310))
             self.client.delete_table(name)
 
         finally:
@@ -148,10 +163,7 @@ class TestClient(KuduTestBase, unittest.TestCase):
         table = self.client.table(self.ex_table)
         session = self.client.new_session()
         for i in range(nrows):
-            op = table.new_insert()
-            op['key'] = i
-            op['int_val'] = i * 2
-            op['string_val'] = 'hello_%d' % i
+            op = table.new_insert((i, i*2, 'hello_%d' % i))
             session.apply(op)
 
         # Cannot apply the same insert twice, C++ client does not indicate an
@@ -166,31 +178,59 @@ class TestClient(KuduTestBase, unittest.TestCase):
         op['key'] = 1
         op['int_val'] = 111
         op['string_val'] = 'updated'
+        # Insert datetime without timezone specified, will be assumed
+        # to be UTC
+        op['unixtime_micros_val'] = datetime.datetime(2016, 10, 30, 10, 12)
         session.apply(op)
 
-        op = table.new_upsert()
-        op['key'] = 2
-        op['int_val'] = 222
-        op['string_val'] = 'upserted'
+        op = table.new_upsert({0: 2,
+                               1: 222,
+                               2: 'upserted'})
         session.apply(op)
         session.flush()
 
         scanner = table.scanner().open()
         rows = dict((t[0], t) for t in scanner.read_all_tuples())
         assert len(rows) == nrows
-        assert rows[1] == (1, 111, 'updated')
-        assert rows[2] == (2, 222, 'upserted')
+        assert rows[1] == (1, 111, 'updated',
+                           datetime.datetime(2016, 10, 30, 10, 12)
+                           .replace(tzinfo=utc))
+        assert rows[2] == (2, 222, 'upserted', None)
 
         # Delete the rows we just wrote
         for i in range(nrows):
-            op = table.new_delete()
-            op['key'] = i
+            op = table.new_delete({'key': i})
             session.apply(op)
         session.flush()
 
         scanner = table.scanner().open()
         assert len(scanner.read_all_tuples()) == 0
 
+    def test_failed_write_op(self):
+        # Insert row
+        table = self.client.table(self.ex_table)
+        session = self.client.new_session()
+        session.apply(table.new_insert({'key': 1}))
+        session.flush()
+
+        # Attempt to insert row again
+        session.apply(table.new_insert({'key': 1}))
+        self.assertRaises(kudu.KuduBadStatus, session.flush)
+
+        # Check errors
+        errors, overflowed = session.get_pending_errors()
+        self.assertFalse(overflowed)
+        self.assertEqual(len(errors), 1)
+        error = errors[0]
+        # This test passes because we currently always return
+        # True.
+        self.assertTrue(error.was_possibly_successful())
+        self.assertEqual(error.failed_op(), 'INSERT int32 key=1')
+
+        # Delete inserted row
+        session.apply(table.new_delete({'key': 1}))
+        session.flush()
+
     def test_session_auto_open(self):
         table = self.client.table(self.ex_table)
         scanner = table.scanner()
@@ -217,7 +257,7 @@ class TestClient(KuduTestBase, unittest.TestCase):
 
     def test_connect_timeouts(self):
         # it works! any other way to check
-        kudu.connect(self.master_host, self.master_port,
+        kudu.connect(self.master_hosts, self.master_ports,
                      admin_timeout_ms=100,
                      rpc_timeout_ms=100)
 
@@ -236,6 +276,100 @@ class TestClient(KuduTestBase, unittest.TestCase):
             assert tserver.hostname() is not None
             assert tserver.port() is not None
 
+    def test_bad_partialrow(self):
+        table = self.client.table(self.ex_table)
+        op = table.new_insert()
+        # Test bad keys or indexes
+        keys = [
+            ('not-there', KeyError),
+            (len(self.schema) + 1, IndexError),
+            (-1, IndexError)
+        ]
+
+        for key in keys:
+            with self.assertRaises(key[1]):
+                op[key[0]] = 'test'
+
+        # Test incorrectly typed data
+        with self.assertRaises(TypeError):
+            op['int_val'] = 'incorrect'
+
+        # Test setting NULL in a not-null column
+        with self.assertRaises(kudu.errors.KuduInvalidArgument):
+            op['key'] = None
+
+    def test_alter_table_rename(self):
+        try:
+            self.client.create_table('alter-rename',
+                                     self.schema,
+                                     self.partitioning)
+            table = self.client.table('alter-rename')
+            alterer = self.client.new_table_alterer(table)
+            table = alterer.rename('alter-newname').alter()
+            self.assertEqual(table.name, 'alter-newname')
+        finally:
+            self.client.delete_table('alter-newname')
+
+    def test_alter_column(self):
+        try:
+            self.client.create_table('alter-column',
+                                     self.schema,
+                                     self.partitioning)
+            table = self.client.table('alter-column')
+            alterer = self.client.new_table_alterer(table)
+            alterer.alter_column('string_val',rename_to='string_val_renamed')
+            table = alterer.alter()
+
+            # Confirm column rename
+            col = table['string_val_renamed']
+
+        finally:
+            self.client.delete_table('alter-column')
+
+    def test_alter_table_add_drop_column(self):
+        table = self.client.table(self.ex_table)
+        alterer = self.client.new_table_alterer(table)
+        alterer.add_column('added-column', type_='int64', default=0)
+        table = alterer.alter()
+
+        # Confirm column was added
+        expected_repr = 'Column(added-column, parent={0}, type=int64)'\
+            .format(self.ex_table)
+        self.assertEqual(expected_repr, repr(table['added-column']))
+
+        alterer = self.client.new_table_alterer(table)
+        alterer.drop_column('added-column')
+        table = alterer.alter()
+
+        # Confirm column has been dropped.
+        with self.assertRaises(KeyError):
+            col = table['added-column']
+
+    def test_alter_table_add_drop_partition(self):
+        # Add Range Partition
+        table = self.client.table(self.ex_table)
+        alterer = self.client.new_table_alterer(table)
+        # Drop the unbounded range partition.
+        alterer.drop_range_partition()
+        # Add a partition from 0 to 100
+        alterer.add_range_partition(
+            lower_bound={'key': 0},
+            upper_bound={'key': 100}
+        )
+        table = alterer.alter()
+        # TODO(jtbirdsell): Once C++ client can list partition schema
+        # then this test should confirm that the partition was added.
+        alterer = self.client.new_table_alterer(table)
+        # Drop the partition from 0 to 100
+        alterer.drop_range_partition(
+            lower_bound={'key': 0},
+            upper_bound={'key': 100}
+        )
+        # Add back the unbounded range partition
+        alterer.add_range_partition()
+        table = alterer.alter()
+
+
 class TestMonoDelta(unittest.TestCase):
 
     def test_empty_ctor(self):
